
import com.ibm.websphere.rest.RESTUtils;
import com.ibm.websphere.rest.RenderUtils;

/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

def _redact(r) {
    def REDACTED_KEYS = [
        'created', 'description', 'id', 'name', 'updated'
    ];

    // re-fetch partial objects
    if (! r.keySet().containsAll(REDACTED_KEYS))
        r = getGroup(r.id);

    // cull attributes
    def owner = "/resources/users/${r.ownerid}"
    r.keySet().retainAll(REDACTED_KEYS);

    // add relationship attributes
    r.owner = owner;
    r.users = getGroupMembers(r.id).collect { it ? "/resources/users/${it.id}" : null }.findAll { it };

    return r
}


/**
 * @success 200 Returns the list of user groups defined on the
 * CloudBurst appliance.  See the example below for a sample of the
 * data returned.
 * @error 403 If the requester does not have sufficient permission
 * to view the list of user groups.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * [
 *    {
 *       "created": 1242962746736,
 *       "description": "This group contains all users in the system",
 *       "id": 1,
 *       "name": "Everyone",
 *       "owner": "/resources/users/1",
 *       "updated": 1242962746736,
 *       "users": [
 *          "/resources/users/1",
 *          "/resources/users/2",
 *          "/resources/users/3"
 *       ]
 *    },
 *    {
 *       "created": 1245422061481,
 *       "description": "Description of the new group",
 *       "id": 2,
 *       "name": "newgroup",
 *       "owner": "/resources/users/1",
 *       "updated": 1245422061481,
 *       "users": [
 *       ]
 *    }
 * ]
 * </pre>
 *
 * <p>See the description of GET /resources/groups/{id} for attribute
 * details.</p>
 */
def onList() 
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = request.json.output[].collect { _redact(it) }
        }]

    RESTUtils.handleOnList([name: request.params.name[]]);
}


/**
 * @success 200 Returns information about a user group defined to the
 * WebSphere CloudBurst appliance.
 * @error 403 If the requester does not have permission to view
 * the user group.
 * @error 404 If the requested user is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * {
 *    "created": 1242962746736,
 *    "description": "This group contains all users in the system",
 *    "id": 1,
 *    "name": "Everyone",
 *    "owner": "/resources/users/1",
 *    "updated": 1242962746736,
 *    "users": [
 *       "/resources/users/1",
 *       "/resources/users/2",
 *       "/resources/users/3"
 *    ]
 * }
 * </pre>
 *
 * <p>A user group has the following attributes:</p>
 *
 * <p>created - Creation time of the user group, as number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 *
 * <p>description - Description of the user group.</p>
 *
 * <p>id - Id of the user group.  This value is numeric and is
 * automatically generated by the appliance.</p>
 *
 * <p>name - Name of the user group.</p>
 *
 * <p>owner - URI of the user that owns this user group.  The URI
 * is relative and should be resolved against the URI of the
 * user group.</p>
 *
 * <p>updated - Time the user group was last updated, as number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 *
 * <p>users - List of URIs of the users that belong to this user
 * group.  The URIs are relative should be resolved against the URI of
 * the user group that contains them.</p>
 */
def onRetrieve()
{
	 def id = request.params.groupsId[].toInteger();
	 if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }];

	 RESTUtils.handleOnRetrieve(id);
}


/**
 * @success 201 The user has been defined and is included in the
 * response body.  The URL of the new user is included in the
 * response's Location header.
 * @error 400 If there are problems parsing the JSON data in the
 * request.
 * @error 403 If the requester does not have sufficient permission
 * to define a new user.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "description": "Description of the new group",
 *    "name": "newgroup"
 * }
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1245422061481,
 *    "description": "Description of the new group",
 *    "id": 2,
 *    "name": "newgroup",
 *    "owner": "/resources/users/1",
 *    "updated": 1245422061481,
 *    "users": [
 *    ]
 * }
 * </pre>
 */
def onCreate()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }];

    RESTUtils.handleOnCreate();
}


/**
 * @success 200 The user group was successfully updated.  The response
 * body contains a JSON representation of the user group's current
 * state.
 * @error 400 If there are problems parsing the JSON data in the
 * request.
 * @error 403 If the requester does not have permission to update
 * the specified user group.
 * @error 404 If the request references a resource that is not
 * defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1245422061481,
 *    "description": "new description",
 *    "id": 2,
 *    "name": "new name",
 *    "owner": "/resources/users/1",
 *    "updated": 1245422061481,
 *    "users": [
 *       "/resources/users/2"
 *    ]
 * } 
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1245422061481,
 *    "description": "new description",
 *    "id": 2,
 *    "name": "new name",
 *    "owner": "/resources/users/null",
 *    "updated": 1245422749173,
 *    "users": [
 *       "/resources/users/2"
 *    ]
 * }
 * </pre>
 */
def onUpdate()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.update = [{ group ->
            def newids = group.users.collect { user ->
                def m = user =~ /\/resources\/users\/(\d+)/;
                if (m)
                    return m.group(1).toInteger();
            }

            if (! newids.every()) {
                logger.FINE { "returning 404 for invalid group.users: ${group.users}" };
                com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08004');
            }

            def oldids = getGroupMembers(group.id).collect { it.id };

            (oldids - newids).each { removeGroupMember(group.id, it) }
            (newids - oldids).each { addGroupMember(group.id, it) }

            group.remove('users');
            return group;
        }, { group ->
            // getting an owner update through the logic in SQLUtils is too
            // difficult for now
            group.remove('owner');
            return group;
        }];

        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }];

        def id = request.params.groupsId[].toInteger();
        RESTUtils.handleOnUpdate(id);
}


/**
 * @success 204 The user group has been deleted.
 * @error 403 If the requester does not have permission to delete
 * the user group.
 * @error 404 If the requested user group is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 */
def onDelete()
{
    def id = request.params.groupsId[].toInteger();
    
    //
    // HACK: 'everyone' group can never be deleted
    //
    if (id == 1)
        return RenderUtils.com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    RESTUtils.handleOnDelete(id);
}
