/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

def _redact(r) {
    def REDACTED_KEYS = [
        'created', 'id', 'gateway', 'name', 'netmask',
        'primarydns', 'secondarydns', 'subnetaddress', 'updated'
    ]

    // attribute mappings
    if (r.containsKey('address'))
        r.subnetaddress = r.address

    // re-fetch partial objects
    if (! r.keySet().containsAll(REDACTED_KEYS)) {
        r = getSubnet(r.id)
        r.subnetaddress = r.address
    }

    // cull attributes
    r.keySet().retainAll(REDACTED_KEYS)

    // add relationship attributes
    r.networks = getNetworks([subnetid: r.id]).collect { "/resources/networks/${it.id}" }

    return r
}


/**
 * @success 200 Returns the list of IP groups defined to CloudBurst.
 * See the example below for a sample of the data returned.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * [
 *    {
 *       "created": 1242243975898,
 *       "gateway": "10.0.0.1",
 *       "id": 2,
 *       "name": "ten",
 *       "netmask": "255.0.0.0",
 *       "networks": ["/resources/networks/1", "/resources/networks/2"],
 *       "primarydns": "10.0.0.2",
 *       "secondarydns": "10.0.0.3",
 *       "subnetaddress": "10.0.0.0",
 *       "updated": 1242738679669
 *    },
 *    {
 *       "created": 1242324839363,
 *       "gateway": "192.168.0.1",
 *       "id": 7,
 *       "name": "192.168.0.0",
 *       "netmask": "255.255.255.0",
 *       "networks": [],
 *       "primarydns": "192.168.0.2",
 *       "secondarydns": "192.168.0.3",
 *       "subnetaddress": "192.168.0.0",
 *       "updated": 1242324839363
 *    }
 * ]
 * </pre>
 *
 * <p>See the description of GET /resources/ipGroups/{id} for
 * attribute details.</p>
 */
def onList()
{
    request.filters.out = [{
        request.json.output = request.json.output[].collect { _redact(it) }
    }]

    restProxy('ipGroups', 'subnets')
}


/**
 * @success 200 Returns information about an IP group defined to the
 * WebSphere CloudBurst appliance.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested IP group is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * {
 *    "created": 1242243975898,
 *    "gateway": "10.0.0.1",
 *    "id": 2,
 *    "name": "ten",
 *    "netmask": "255.0.0.0",
 *    "networks": ["/resources/networks/1", "/resources/networks/2"],
 *    "primarydns": "10.0.0.2",
 *    "secondarydns": "10.0.0.3",
 *    "subnetaddress": "10.0.0.0",
 *    "updated": 1242738679669
 * }
 * </pre>
 *
 * <p>An IP group has the following attributes:</p>
 *
 * <p>created - Creation time of the IP group, as the number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 *
 * <p>gateway - Default gateway associated with the IP group,
 * represented as a string in dotted decimal notation ('192.168.98.1',
 * for example).</p>
 *
 * <p>id - Id of the IP group.  This value is numeric and is
 * automatically generated by the appliance.</p>
 *
 * <p>name - Display name associated with this IP group.  This value
 * is a string.</p>
 *
 * <p>netmask - Network mask associated with the IP group's subnet
 * address, represented as a string in dotted decimal notation
 * ('255.255.255.0', for example).</p>
 *
 * <p>networks - List of URIs of the networks associated with the
 * IP group.  The URIs are relative should be resolved against the URI
 * of the IP group that contains them.</p>
 *
 * <p>primarydns - Primary DNS server used for the IP group,
 * represented as a string in dotted decimal notation ('192.168.98.2',
 * for example).</p>
 *
 * <p>secondarydns - Secondary DNS server used for the IP group,
 * represented as a string in dotted decimal notation ('192.168.98.3',
 * for example).</p>
 *
 * <p>subnetaddress - Subnet address associated with the IP group,
 * represented as a string in dotted decimal notation ('192.168.98.0',
 * for example).</p>
 *
 * <p>updated - Time the IP group was last updated, as number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 */
def onRetrieve()
{
    request.filters.out = [{
        request.json.output = _redact(request.json.output[])
    }]

    restProxy('ipGroups', 'subnets')
}


/**
 * @success 201 The IP group has been created and is included in the
 * response body.  The URL of the new IP group is included in the
 * response's Location header.
 * @error 400 If there are problems parsing the JSON data in the request.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "gateway": "10.0.0.1",
 *    "name": "ten",  (optional, defaults to subnetaddress)
 *    "netmask": "255.0.0.0",
 *    "primarydns": "10.0.0.2",
 *    "secondarydns": "10.0.0.3",  (optional, no default)
 *    "subnetaddress": "10.0.0.0"
 * }
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1242243975898,
 *    "gateway": "10.0.0.1",
 *    "id": 2,
 *    "name": "ten",
 *    "netmask": "255.0.0.0",
 *    "networks": [],
 *    "primarydns": "10.0.0.2",
 *    "secondarydns": "10.0.0.3",
 *    "subnetaddress": "10.0.0.0",
 *    "updated": 1242738679669
 * }
 * </pre>
 */
def onCreate()
{
    request.filters.in = [{
        if (it.subnetaddress)
            it.address = it.subnetaddress

        it.remove('subnetaddress')
        return it
    }]

    request.filters.out = [{
        request.headers.out.Location = request.headers.out.Location[0].replace('subnets', 'ipGroups');
        request.json.output = _redact(request.json.output[])
    }]

    restProxy('ipGroups', 'subnets')
}


/**
 * @success 200 The IP group was successfully updated.  The response
 * body contains a JSON representation of the IP group's current
 * state.
 * @error 400 If there are problems parsing the JSON data in the
 * request.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested IP group is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "gateway": "10.128.1.2",
 *    "name": "new name",
 *    "netmask": "255.128.0.0",
 *    "networks": [ "/resources/networks/3" ]
 *    "primarydns": "10.128.2.3",
 *    "secondarydns": "10.128.3.4",
 *    "subnetaddress": "10.128.0.0"
 * }
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1242243975898,
 *    "gateway": "10.128.1.2",
 *    "id": 2,
 *    "name": "new name",
 *    "netmask": "255.128.0.0",
 *    "networks": [ "/resources/networks/3" ]
 *    "primarydns": "10.128.2.3",
 *    "secondarydns": "10.128.3.4",
 *    "subnetaddress": "10.128.0.0",
 *    "updated": 1242738679669
 * }
 * </pre>
 */
def onUpdate()
{
    request.filters.in = [{
        if (it.subnetaddress)
            it.address = it.subnetaddress

        it.remove('subnetaddress')
        return it
    }]

    request.filters.update = [{
        def newids = it.networks.collect {
            def m = it =~ /\/resources\/networks\/(\d+)/
            if (m)
                return m.group(1).toInteger()
        }

        if (! newids.every() ||
            ! handleOnUpdateHasMany('networks', 'subnetid', it.id, newids))
            com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08004');

        it.remove('networks')
        return it
    }]

    request.filters.out = [{
        request.json.output = _redact(request.json.output[])
    }]

    restProxy('ipGroups', 'subnets')
}


/**
 * @success 204 The IP group has been deleted.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested IP group is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 */
def onDelete()
{
    restProxy('ipGroups', 'subnets')
}
