import com.ibm.websphere.rest.RESTUtils;
import com.ibm.websphere.rest.RenderUtils;

/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;
import com.ibm.websphere.rainmaker.logging.MessageUtils;

def _redact(r) {
    r.keySet().retainAll([
        'created', 'currentmessage', 'currentstatus', 'id', 'ipaddress',
        'updated'
    ]);

    r.currentmessage_text = r.currentmessage ? MessageUtils.message(user.lang[], r.currentmessage) : null;
    r.currentstatus_text = MessageUtils.message(user.lang[], r.currentstatus);

    return r
}


/**
 * @success 200 Returns the list of IPs defined for an IP group .
 * See the example below for a sample of the data returned.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * [
 *    {
 *       "created": 1245009489214,
 *       "currentmessage": null,
 *       "currentmessage_text": null,
 *       "currentstatus_text": "Inactive",
 *       "currentstatus": "RM01017",
 *       "id": 1,
 *       "ipaddress": "10.1.2.3",
 *       "updated": 1245042922441
 *    },
 *    {
 *       "created": 1245009489294,
 *       "currentmessage": null,
 *       "currentmessage_text": null,
 *       "currentstatus": "RM01017",
 *       "currentstatus_text": "Inactive",
 *       "id": 2,
 *       "ipaddress": "10.1.2.4",
 *       "updated": 1245009489294
 *    }
 * ]
 * </pre>
 *
 * <p>See the description of GET /resources/ipGroups/{id}/ips/{id} for
 * attribute details.</p>
 */
def onList()
{
    if (!com.ibm.websphere.rest.RESTUtils.authorized())
        return RenderUtils.com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = request.json.output[].collect { _redact(it) }
        }]

    def subnetID = null;
    try {
        subnetID = (request.params.subnetsId[] ?: request.params.ipGroupsId[]).toInteger();
    }
    catch (Exception e) {
        // No subnet specified, list all
        return RESTUtils.handleOnList();
    }

    def filters = ["subnetid": subnetID];
    ArrayList ips = getIPs(filters);

    request.view = RenderUtils.getRequestedView();
    request.json.output = ips;

    if (request.filters.out[])
        request.filters.out[].each { it() }

    render();
}


/**
 * @success 200 Returns information about an IP address defined to the
 * WebSphere CloudBurst appliance.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested IP address is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * {
 *    "created": 1245009489214,
 *    "currentmessage": null,
 *    "currentmessage_text": null,
 *    "currentstatus": "RM01017",
 *    "currentstatus_text": "Inactive",
 *    "id": 1,
 *    "ipaddress": "10.1.2.3",
 *    "updated": 1245042922441,
 * }
 * </pre>
 *
 * <p>An IP group has the following attributes:</p>
 *
 * <p>created - Creation time of the IP, as number of milliseconds
 * since midnight, January 1, 1970 UTC.  This value is numeric and is
 * automatically generated by the appliance.</p>
 *
 * <p>currentmessage - Message associated with the current status of
 * the IP.  This string value is generated by the appliance.</p>
 *
 * <p>currentmessage_text - Textual representation of currentmessage.
 * This is a string representation of currentmessage in the
 * requester's preferred language and is automatically generated by
 * the appliance.</p>
 *
 * <p>currentstatus - Current status of the IP.  A string constant
 * representing the current status of the IP address.  This value
 * is automatically generated by the appliance.</p>
 *
 * <p>currentstatus_text - Textual representation of currentstatus.
 * This is a string representation of currentstatus in the
 * requester's preferred language and is automatically generated by
 * the appliance.</p>
 *
 * <p>id - Id of the IP address.  This numeric value is automatically
 * generated by the appliance.</p>
 *
 * <p>ipaddress - IP address associated with this IP.  The IP address
 * must be unique and must belong to the IP group under which this IP
 * is defined.  This value is a string in dotted-decimal notation
 * ('10.1.2.3', for example).</p>
 *
 * <p>updated - Time the IP address was last updated, as the number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 */
def onRetrieve()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }]

    RESTUtils.handleOnRetrieve(request.params.ipsID[]);
}


/**
 * @success 201 The IP group has been created and is included in the
 * response body.  The URL of the new IP group is included in the
 * response's Location header.
 * @error 400 If there are problems parsing the JSON data in the request
 * or if the IP group is specified incorrectly.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "addresses": ["10.1.2.3", "10.1.2.4"]
 * }
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * [
 * ]
 * </pre>
 */
def onCreate()
{
    if (!com.ibm.websphere.rest.RESTUtils.authorized())
        return RenderUtils.com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

    def subnetID = null;
    try {
        subnetID = (request.params.subnetsId[] ?: request.params.ipGroupsId[]).toInteger();
    }
    catch (Exception e) {
        return RenderUtils.com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', e);
    }

    def ips = null;

    try {
        def addresslist = Json.decode(request.input[]);
        ips = addresslist.addresses;
    }
    catch (Exception e) {
        return RenderUtils.com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', e);
    }

    def ipids = new ArrayList();
    
    if (ips != null && ips.size() > 0) {

        // Validate the first and last IP.  If either of these is not valid, return an error.
        try {
            isValidIP(ips.get(0), false);
            ipCanBelongToSubnet(ips.get(0), subnetID);
            if (ips.size() > 1) {
                isValidIP(ips.get(ips.size() - 1), false);
                ipCanBelongToSubnet(ips.get(ips.size() - 1), subnetID);
            }
        } catch (Exception e) {
            return RenderUtils.com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08022', e);
        }

        // Add each IP.  All but the 1st and last will have been programmatically 
        // generated in the UI, so invalid entries are just ignored.
        ips.each {
            try {
                HashMap ipHash = new HashMap();
                ipHash.ipaddress = it;
                ipHash.subnetid = subnetID;
                createIP(ipHash);
            } catch (Exception e) {
                // Ignore any failures, IP is just not created
            }
        }
    }

    request.status = 201;
    
    // Empty ArrayList is intentionally returned as ids not useful
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
    request.json.output = ipids;
    render();
}


// MJB - as far as I can tell, IPs have no updatable attributes
def onUpdate()
{
    RESTUtils.handleOnUpdate();
}


/**
 * @success 204 The IP address has been deleted.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested IP address is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 */
def onDelete()
{
    RESTUtils.handleOnDelete(request.params.ipsID[]);
}
