/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.websphere.rainmaker.rest.log.LogViewer;

import com.ibm.websphere.users.Permissions;

/**
 * @success 200 If log viewing for the specified log file was
 * successfully initialized.  The Location header in the response
 * contains a URL that can be queried to view contents of the log
 * file.
 * @error 403 If the requester does not have the appliance admin role.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @example
 * <p>This REST API call initializes log viewing for a specific log
 * file.  The Location header in the response contains a URL that can
 * be subsequently queried to fetch new contents of the specified
 * file.</p>
 *
 * <p>Request:</p>
 *
 * <pre>
 * POST /resources/logViewerMgr?logfile=trace/trace.log
 * </pre>
 *
 * <p>Response headers:</p>
 *
 * <pre>
 * Location: https://myappliance.mycompany.com/resources/logViewerMgr/trace_%5E_trace.log
 * </pre>
 */
def onCreate(){
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

	def logFile = request.params.logfile[];
	logger.FINEST{
		"logFile name " + logFile
	}

	request.status = 200;  //IE does not handle 204 -- returns 1223 and unable to get headers
	def logViewer = new LogViewer(logFile);
	if(!zcontains("/user/logViewer")){
		user.logViewer = [:];
	}
	
	logFile = logFile.replace('/', '_^_');	// encode the directory pathing since zero's rest does not pass all pathing
	
	zput("/user/logViewer" + "#" + logFile, logViewer );
	logger.FINE{
		"storing logViewer in session under " + "/user/logViewer" + "#" + logFile
	}
	
	request.headers.out.Location = zero.core.utils.URIUtils.getRequestedUri(false) + "/" + URLEncoder.encode(logFile, "UTF-8");
	logger.FINEST{
		"logFile location " + request.headers.out.Location[]
	}

	
}


/**
 * @success 200 if content from the log file is included in the
 * output.
 * @success 204 if the specified startingPoint and lineCount do not
 * include any content from the log file.
 * @error 403 If the requester does not have the appliance admin role.
 * @error 404 If the specified log viewing has not been initialized
 * correctly.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request:</p>
 *
 * <pre>
 * GET /resources/logViewerMgr/trace_%5E_trace.log?startingPoint=0&lineCount=3
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "NEXT_CHUNK": 214,
 *    "TAIL_CONTENT": "************ Start Display Current Environment ************\nWebSphere CloudBurst version number is 1.0.0.1-11776\nJava Version = J2RE 1.6.0 IBM J9 2.4 Linux x86-32 jvmxi3260-20090215_29883 &#40;JIT enabled, AOT enabled&#41;"
 * }
 * </pre>
 *
 * <p>The TAIL_CHUNK entry in the response contains contents of
 * the log file; the NEXT_CHUNK value in the response can be used
 * as the startingPoint in the next request to retrieve subsequent
 * content from the log file.</p>
 */
def onRetrieve(){
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

	def client = zget("/user/logViewer" + "#" + (String)request.params.logViewerMgrId[] ) ;

    if (client == null){
    	client = new LogViewer(request.params.logViewerMgrId[].replace('_^_', '/'));
    	if(request.params.standalone[]?.toBoolean()) {
    		logger.FINEST { "No log viewer found in session, creating a single-use viewer that will not retain position." };
    	}else{	// should not happen but we have seen it so adding this escape route
    		logger.FINE { "No log viewer found in session, storing newly created logViewer in session " };
    		if(!zcontains("/user/logViewer")){
    			user.logViewer = [:];
    		}
    		zput("/user/logViewer" + "#" + (String)request.params.logViewerMgrId[], client ) ;
    	}
    }

	def lineCount = request.params.lineCount[];
	def startingPoint = zget("/request/params/startingPoint", -1);

	logger.FINEST{
		"id: [${request.params.logViewerMgrId[]}] client:[${client}]  lineCount:[${lineCount}] startingPoint:[$startingPoint}]"
	}
	
	def output;
	if(client != null){
		try{
			output = client.getResponseContent(Integer.valueOf(lineCount), Integer.valueOf(startingPoint) );
		}catch (Exception e){
			com.ibm.websphere.rest.RenderUtils.renderError(503, 'RM08056', e);
			return;
	
		}
	}else{
		com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08056', new Exception("Unable to find logViewer "+ (String)request.params.logViewerMgrId[]));
		return;
	}

	
	if(output != null){
		logger.FINEST{
			"log output exists. writing to client"
		}
		request.status = 200;
		request.view="JSON"
		request.json.output=output
		render()
	}else{
		logger.FINEST{
			"no log output."
		}
		request.status = 204;
	}
}

def onDelete(){
	//TBD
}
