import com.ibm.websphere.rest.RESTUtils;
import com.ibm.websphere.rainmaker.logging.MessageUtils;

/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

def _redact(r)
{
    def REDACTED_KEYS = [
        'created', 'currentmessage', 'id', 'name', 'updated',
        'vlan'
    ];

    // re-fetch partial objects
    if (! r.keySet().containsAll(REDACTED_KEYS))
        r = getNetwork(r.id);
    
    // cull attributes
    def hypervisoruri = "/resources/hypervisors/${getPhysicalMachineHypervisor([ id: r.physicalmachineid ]).id}";
    def ipgroupuri = r.subnetid && r.subnetid > 0 ? "/resources/ipgroups/${r.subnetid}" : null;
    r.keySet().retainAll(REDACTED_KEYS);

    // add translated attributes
    r.currentmessage_text = r.currentmessage ? MessageUtils.message(user.lang[], r.currentmessage) : null;
    
    // add relationship attributes
    r.hypervisor = hypervisoruri;
    r.ipgroup = ipgroupuri;

    return r;
}


/**
 * @success 200 Returns the list of networks defined on the
 * CloudBurst appliance.  See the example below for a sample of the
 * data returned.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * [
 *    {
 *       "created": 1245009567989,
 *       "currentmessage": null,
 *       "currentmessage_text": null,
 *       "hypervisor": "/resources/hypervisors/1",
 *       "id": 1,
 *       "ipgroup": null,
 *       "name": "Public VM Network",
 *       "updated": 1245009567989,
 *       "vlan": null
 *    },
 *    {
 *       "created": 1245009568100,
 *       "currentmessage": null,
 *       "currentmessage_text": null,
 *       "hypervisor": "/resources/hypervisors/1",
 *       "id": 2,
 *       "ipgroup": "/resources/ipgroups/1",
 *       "name": "VM Network",
 *       "updated": 1245009586180,
 *       "vlan": null
 *    }
 * ]
 * </pre>
 *
 * <p>See the description of GET /resources/networks/{id} for
 * attribute details.</p>
 */
def onList()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = request.json.output[].collect { _redact(it) }
        }]

    RESTUtils.handleOnList() { networks ->
        networks.each { network ->
            network.hypervisorid = getPhysicalMachineHypervisor([ id: network.physicalmachineid ])?.id;
        }
    };
}


/**
 * @success 200 Returns information about a network defined to the
 * WebSphere CloudBurst appliance.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested network is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * {
 *    "created": 1245009568100,
 *    "currentmessage": null,
 *    "currentmessage_text": null,
 *    "hypervisor": "/resources/hypervisors/1",
 *    "id": 2,
 *    "ipgroup": "/resources/ipgroups/1",
 *    "name": "VM Network",
 *    "updated": 1245009586180,
 *    "vlan": null
 * }
 * </pre>
 *
 * <p>A network has the following attributes:</p>
 *
 * <p>created - Creation time of the network, as number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 *
 * <p>currentmessage - Message associated with the current status of
 * the network.  This string value is generated by the appliance.</p>
 *
 * <p>currentmessage_text - Textual representation of currentmessage.
 * This is a string representation of currentmessage in the
 * requester's preferred language and is automatically generated by
 * the appliance.</p>
 *
 * <p>hypervisor - URI of the hypervisor to which this network
 * belongs.  The URI is relative and should be resolved against the
 * URI of the network.</p>
 *
 * <p>id - Id of the network.  This value is numeric and is
 * automatically generated by the appliance.</p>
 *
 * <p>ipgroup - URI of the ipgroup to which this network has been
 * assigned.  The URI is relative and should be resolved against the
 * URI of the network.</p>
 *
 * <p>name - Name of the network, as reported by the hypervisor.  This
 * value is a string.</p>
 *
 * <p>updated - Time the network was last updated, as number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 *
 * <p>vlan - VLAN associated with this network.  This value is an
 * integer between 0 and 4095, inclusive.</p>
 */
def onRetrieve()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }]

    RESTUtils.handleOnRetrieve(request.params.networksId[]) { network ->
        network.hypervisorid = getPhysicalMachineHypervisor([ id: network.physicalmachineid ])?.id;
    };
}


// TODO - we don't allow the user to create networks do we?
def onCreate()
{
    RESTUtils.handleOnCreate();
}


/**
 * @success 200 The network was successfully updated.  The response
 * body contains a JSON representation of the network's current state.
 * @error 400 If there are problems parsing the JSON data in the
 * request.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the request references a resource that is not
 * defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1245009568100,
 *    "currentmessage": null,
 *    "currentmessage_text": null,
 *    "hypervisor": "/resources/hypervisors/1",
 *    "id": 2,
 *    "ipgroup": "/resources/ipgroups/1",
 *    "name": "new network name",
 *    "updated": 1245009586180,
 *    "vlan": 17
 * }
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1245009568100,
 *    "currentmessage": null,
 *    "currentmessage_text": null,
 *    "hypervisor": "/resources/hypervisors/1",
 *    "id": 2,
 *    "ipgroup": "/resources/ipgroups/1",
 *    "name": "new network name",
 *    "updated": 1245009597319,
 *    "vlan": 17
 * }
 * </pre>
 */
def onUpdate()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion()) {
        request.filters.update = [{
            if (it.containsKey('hypervisor')) {
                def m = it.hypervisor =~ /\/resources\/hypervisors\/(\d+)/
                if (! m)
                    com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08004');
                else
                    it.physicalmachineid = getHypervisorsPhysicalMachinesForHypervisor(m.group(1).toInteger())[0].physicalmachineid;

                it.remove('hypervisor');
            }

            if (it.containsKey('ipgroup')) {
                if (it.ipgroup) {
                    def m = it.ipgroup =~ /\/resources\/ipgroups\/(\d+)/
                    if (! m)
                        com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08004');
                    else
                        it.subnetid = m.group(1).toInteger();
                }
                else
                    it.subnetid = 0;

                it.remove('ipgroup');
            }

            it.remove('currentmessage_text');
            return it
        }]

        request.filters.out = [{
            request.json.output = _redact(request.json.output[])
        }]
    }

    RESTUtils.handleOnUpdate();
}


/**
 * @success 204 The network has been deleted.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested network is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 */
def onDelete()
{
    RESTUtils.handleOnDelete(request.params.networksId[]);
}
