/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

def _redact(r)
{
    def REDACTED_KEYS = [
        'created', 'hypervisorstorageid', 'id', 'name', 'updated'
    ];

    // re-fetch partial objects
    if (! r.keySet().containsAll(REDACTED_KEYS))
        r = getStorage(r.id);
    
    // cull attributes
    r.keySet().retainAll(REDACTED_KEYS);

    // add relationship attributes
    def pmstorages = getPhysicalMachinesStorageForStorage(r.id);
    def hypervisors = pmstorages.collect { getPhysicalMachineHypervisor([id: it.physicalmachineid]) };
    r.hypervisors = hypervisors.collect { "/resources/hypervisors/${it.id}" };

    return r;
}

/**
 * @success 200 Returns the list of storage defined on the
 * CloudBurst appliance.  See the example below for a sample of the
 * data returned.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * [
 *    {
 *       "created": 1245009567366,
 *       "hypervisors": [
 *          "/resources/hypervisors/1"
 *       ],
 *       "hypervisorstorageid": "https://myhypervisor.mycom/sdk#Datastore#497e24f5-ab7b1019-30a5-001a643670e6",
 *       "id": 1,
 *       "name": "myhypervisor:localstorage1",
 *       "updated": 1245009567366
 *    },
 *    {
 *       "created": 1245009567758,
 *       "hypervisors": [
 *          "/resources/hypervisors/1"
 *       ],
 *       "hypervisorstorageid": "https://myhypervisor.mycom/sdk#Datastore#49824d1c-f0c6ea83-f3e0-001a643670e6"
 *       "id": 2,
 *       "name": "myhypervisor:sanstorage2",
 *       "updated": 1245009567758,
 *    }
 * ]
 * </pre>
 *
 * <p>See the description of GET /resources/storage/{id} for attribute
 * details.</p>
 */
def onList()
{
    if (!com.ibm.websphere.rest.RESTUtils.authorized())
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = request.json.output[].collect { _redact(it) }
        }]

    def filters = [
        name: request.params.name[], 
        count: request.params.count[], 
        page: request.params.page[], 
        order: request.params.order[] 
    ];
    
    def resources = null;
    
    try
    {
        resources = getStorageCollection(filters);
        
        def relevance = request.params.relevance[];
        
        if (relevance && filters[relevance])
            resources = sortByRelevance(resources, relevance, filters[relevance]);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08000', t);
    }
    
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
    request.json.output = resources;

    if (request.filters.out[])
        request.filters.out[].each { it() }

    render();
}


/**
 * @success 200 Returns information about storage defined to the
 * WebSphere CloudBurst appliance.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested storage is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <pre>
 * {
 *    "created": 1245009567758,
 *    "hypervisors": [
 *       "/resources/hypervisors/1"
 *    ],
 *    "hypervisorstorageid": "https://myhypervisor.mycom/sdk#Datastore#49824d1c-f0c6ea83-f3e0-001a643670e6"
 *    "id": 2,
 *    "name": "myhypervisor:sanstorage2",
 *    "updated": 1245009567758,
 * }
 * </pre>
 *
 * <p>Storage has the following attributes:</p>
 *
 * <p>created - Creation time of the storage, as number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 *
 * <p>hypervisors - List of URIs of the hypervisors associated with
 * this storage.  The URIs are relative should be resolved against the
 * URI of the storage that contains them.</p>
 *
 * <p>hypervisorstorageid - Identifier used by hypervisors to identify
 * this storage, automatically determined.</p>
 *
 * <p>id - Id of the storage.  This value is numeric and is
 * automatically generated by the appliance.</p>
 *
 * <p>name - Name of the storage, as reported by the hypervisor.  This
 * value is a string.</p>
 *
 * <p>updated - Time the storage was last updated, as number of
 * milliseconds since midnight, January 1, 1970 UTC.  This value is
 * numeric and is automatically generated by the appliance.</p>
 */
def onRetrieve()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion())
        request.filters.out = [{
            request.json.output = _redact(request.json.output[]);
        }]

    handleOnRetrieve();
}


// TODO - we don't allow the user to create storage do we?
def onCreate()
{
    handleOnCreate();
}


/**
 * @success 200 The storage was successfully updated.  The response
 * body contains a JSON representation of the storage's current state.
 * @error 400 If there are problems parsing the JSON data in the
 * request.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the request references a resource that is not
 * defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 * @format application/json
 * @example
 * <p>Request JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1245009567758,
 *    "hypervisors": [
 *       "/resources/hypervisors/1"
 *    ],
 *    "hypervisorstorageid": "https://myhypervisor.mycom/sdk#Datastore#49824d1c-f0c6ea83-f3e0-001a643670e6"
 *    "id": 2,
 *    "name": "new name",
 *    "updated": 1245009567758,
 * }
 * </pre>
 *
 * <p>Response JSON:</p>
 *
 * <pre>
 * {
 *    "created": 1245009567758,
 *    "hypervisors": [
 *       "/resources/hypervisors/1"
 *    ],
 *    "hypervisorstorageid": "https://myhypervisor.mycom/sdk#Datastore#49824d1c-f0c6ea83-f3e0-001a643670e6"
 *    "id": 2,
 *    "name": "new name",
 *    "updated": 1245009593409,
 * }
 * </pre>
 */
def onUpdate()
{
    if (com.ibm.websphere.rest.RESTUtils.externalApiVersion()) {
        request.filters.update = [{
            def pmids = it.hypervisors.collect {
                def m = it =~ /\/resources\/hypervisors\/(\d+)/
                if (m) {
                    def hyppms = getHypervisorsPhysicalMachinesForHypervisor(m.group(1).toInteger())
                    return hyppms ? hyppms[0].physicalmachineid : null;
                }
            }

            if (! pmids.every() ||
                ! handleOnUpdateManyMany('storage', it.id, 'physicalmachines', pmids))
                com.ibm.websphere.rest.RenderUtils.renderError(404, 'RM08004');

            return it
        }]

        request.filters.out = [{
            request.json.output = _redact(request.json.output[])
        }]
    }

    handleOnUpdate();
}


/**
 * @success 204 The storage has been deleted.
 * @error 403 If the requester has not been assigned the cloud
 * administration role.
 * @error 404 If the requested storage is not defined.
 * @error 500 If the WebSphere CloudBurst appliance encountered an
 * internal error while processing the request.
 */
def onDelete()
{
    handleOnDelete();
}


def onPostMember()
{
    if (zcontains('/request/params/cleanup')) {
        if (!com.ibm.websphere.rest.RESTUtils.authorized())
            return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

        cleanupTask = [
            name: "Cleanup storage ${request.params.storageId[]}",
            event: 'cleanup-storage',
            actiontime: System.currentTimeMillis(),
            priority: -1,
            disk: 0
        ];

        createTask(cleanupTask, [ storageid: request.params.storageId[].toInteger() ]);

        request.status = 202;
    }
}
